/* nfile.c -- a diskette archiving program
   John T. Cockerham
   This program compares two file in two paths and copies 
   files from one path to the other so that the most recent is 
   present in both locations.

   If only one argument is present on the command line,
   then the current path is assumed to be one of the paths.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <direct.h>
#include <dos.h>
#include <process.h>


char first_path    [_MAX_PATH];
char second_path   [_MAX_PATH];
int  file_count    = 0;

char tail(char * string); 
void nfile_copy (char * source_path, char * dest_path);

void main (int argc, char *argv[])
{
  char  c;

  /* Check the number of arguments. Supply command syntax
     if requested by the user */

  if (argc == 2 && (!strcmp(argv[1], "/?")
      || !strcmp(argv[1], "-?"))){
    printf("Use: nfile {<path1>} <path2>\n");
    exit (0);
  }

  switch (argc) {
    case 1:  /* No arguments on command line */
      printf("Use: nfile {<path1>} <path2>\n");
      exit (0);

    case 2:
    /* If the argument count is two, then the user wants 
       the current path as one path */
      getcwd(first_path,    _MAX_PATH);
      strcpy(second_path,   argv[1]);
      break;

    default:
    /* Otherwise the user is specifying both paths */
      strcpy (first_path,   argv[1]);
      strcpy (second_path,  argv[2]);
  }

  /* Upper case the path names */
  _strupr(first_path);
  _strupr(second_path);

  /* If the source or dest does not end with \ then
     append a \ for the path name and a *.* */
  if ((c = tail(first_path))  != '\\') 
     strcat(first_path, "\\*.*");
 
  if ((c = tail(second_path)) != '\\') 
     strcat(second_path,  "\\*.*");
     
  /* Copy files out date from the first to the second
     path and vice versa */
  nfile_copy(first_path, second_path);
  nfile_copy(second_path, first_path);

  if (file_count)
    printf("NFILE: Total of %3i file(s) copied.\n", file_count);
  exit(0);
}
/* This function returns the last character of a string */
char tail(char * string)
{
  return (*(string + strlen(string) - 1));
}

void nfile_copy (char * source_path, char * dest_path)
{

  char template [_MAX_PATH];
  char * end_path;
  struct _find_t  f_source;
  struct _find_t  f_dest;
  
  if (_dos_findfirst(dest_path, _A_NORMAL, &f_dest)) {
    printf("No files in directory %s.\n", dest_path);
  } else {
      strcpy(template, source_path);
      end_path = strrchr(template, '\\') + sizeof(char);
      do {
         /* We have found a file in the dest directory
            Use the find file call to see if it exists in the
            source directory */
         strcpy(end_path, f_dest.name);
         if (!_dos_findfirst(template, _A_NORMAL, &f_source)) {
           if ((f_dest.wr_date == f_source.wr_date
               && f_dest.wr_time <  f_source.wr_time)
               || f_dest.wr_date <  f_source.wr_date) {
                  /* The file in the dest directory is out
                     date with respect to the source. Copy it.*/
             printf("Copying %s to %s.\n", template, dest_path);
             ++file_count;
             _spawnlp(_P_WAIT, "xcopy", "xcopy", 
               template, dest_path, NULL);
           }
         }  
      } while (!_dos_findnext(&f_dest));
  }
}
